import { hexToRGB } from '../../util.js';
import { MermaidDiagram } from './mermaid.js';

export class SequenceDiagram extends MermaidDiagram {
  constructor(...args) {
    super(...args);
    this.activations = [];
    this.sections = [];

    this.log('sequenceDiagram', false);
  }

  async log(...args) {
    this.sections.forEach(async (section, index) => {
      const {
        empty, r, g, b,
      } = section;
      if (empty) {
        section.empty = false;
        await super.log(`rect rgb(${r}, ${g}, ${b}) # ${index}`, false);
      }
    });
    return super.log(...args);
  }

  activate(name) {
    this.log(`activate ${name}`, false);
    this.activations.push(name);
  }

  async deactivate(name) {
    const index = this.activations.findLastIndex((n) => n === name);
    if (index === -1) throw new Error(`${name} does not appear to be active!`);
    this.activations.splice(index, 1);
    await this.log(`deactivate ${name}`);
  }

  getDeactivationsText() {
    const text = Array.from(this.activations).reverse().reduce((str, name) => str += `deactivate ${name}\n`, '');
    return text;
  }

  async startSection(color = '#08252c') {
    let { r, g, b } = hexToRGB(color);
    for (let i = 0; i < this.sections.length; i++) {
      r += (0xff - r) / 16;
      g += (0xff - g) / 16;
      b += (0xff - b) / 16;
    }
    this.sections.push({
      empty: true, r, g, b,
    });
  }

  async endSection() {
    const section = this.sections.pop();
    if (section && !section.empty) {
      await this.log('end');
    }
  }

  getSectionEndText() {
    if (this.sections[this.sections.length - 1]?.empty) {
      this.sections.pop();
    }
    return this.sections.map(() => 'end\n').join('');
  }

  getText() {
    let text = super.getText();
    if (!text.endsWith('\n')) text = `${text}\n`;
    text += this.getDeactivationsText();
    text += this.getSectionEndText();
    return text;
  }
}
